/*
  UTFT.cpp - Multi-Platform library support for Color TFT LCD Boards
  Copyright (C)2015 Rinky-Dink Electronics, Henning Karlsen. All right reserved
  
  This library is the continuation of my ITDB02_Graph, ITDB02_Graph16
  and RGB_GLCD libraries for Arduino and chipKit. As the number of 
  supported display modules and controllers started to increase I felt 
  it was time to make a single, universal library as it will be much 
  easier to maintain in the future.

  Basic functionality of this library was origianlly based on the 
  demo-code provided by ITead studio (for the ITDB02 modules) and 
  NKC Electronics (for the RGB GLCD module/shield).

  This library supports a number of 8bit, 16bit and serial graphic 
  displays, and will work with both Arduino, chipKit boards and select 
  TI LaunchPads. For a full list of tested display modules and controllers,
  see the document UTFT_Supported_display_modules_&_controllers.pdf.

  When using 8bit and 16bit display modules there are some 
  requirements you must adhere to. These requirements can be found 
  in the document UTFT_Requirements.pdf.
  There are no special requirements when using serial displays.

  You can find the latest version of the library at 
  http://www.RinkyDinkElectronics.com/

  This library is free software; you can redistribute it and/or
  modify it under the terms of the CC BY-NC-SA 3.0 license.
  Please see the included documents for further information.

  Commercial use of this library requires you to buy a license that
  will allow commercial use. This includes using the library,
  modified or not, as a tool to sell products.

  The license applies to all part of the library including the 
  examples and tools supplied with the library.
*/

#include "UTFT.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/delay.h>
#include <inttypes.h>
#include <stdbool.h>
#include <stdio.h>
#include "SPILibrary.h"
#include "SPI.h"
#include "HW_AVR_defines.h"
#include "Adafruit_ILI9341.h"
#include "Arduino.h"


#define bitmapdatatype const unsigned int*

bool hwSPI;
uint8_t mySPCR;
int8_t  _cs, _dc, _rst, _mosi, _miso, _sclk;
volatile uint8_t *mosiport, *clkport, *dcport, *rsport, *csport;
uint8_t  mosipinmask, clkpinmask, cspinmask, dcpinmask;


UTFT::UTFT()
{
}

// UTFT::UTFT(byte model, int RS, int WR, int CS, int RST, int SER)
// { 
// 	word	dsx[] = {239, 239, 239, 239, 239, 239, 175, 175, 239, 127, 127, 239, 271, 479, 239, 239, 239, 0, 0, 239, 479, 319, 239, 175, 127, 239, 239, 319, 319, 799, 127, 127};
// 	word	dsy[] = {319, 399, 319, 319, 319, 319, 219, 219, 399, 159, 127, 319, 479, 799, 319, 319, 319, 0, 0, 319, 799, 479, 319, 219, 159, 319, 319, 479, 479, 479, 159, 159};
// 	byte	dtm[] = {16, 16, 16, 8, 8, 16, 8, SERIAL_4PIN, 16, SERIAL_5PIN, SERIAL_5PIN, 16, 16, 16, 8, 16, LATCHED_16, 0, 0, 8, 16, 16, 16, 8, SERIAL_5PIN, SERIAL_5PIN, SERIAL_4PIN, 16, 16, 16, SERIAL_5PIN, SERIAL_5PIN};
// 
// 	disp_x_size =			dsx[model];
// 	disp_y_size =			dsy[model];
// 	display_transfer_mode =	dtm[model];
// 	display_model =			model;
// 
// 	__p1 = RS;
// 	__p2 = WR;
// 	__p3 = CS;
// 	__p4 = RST;
// 	__p5 = SER;
// 
// 	if (display_transfer_mode == SERIAL_4PIN)
// 	{
// 		display_transfer_mode=1;
// 		display_serial_mode=SERIAL_4PIN;
// 	}
// 	if (display_transfer_mode == SERIAL_5PIN)
// 	{
// 		display_transfer_mode=1;
// 		display_serial_mode=SERIAL_5PIN;
// 	}
// 
// 	if (display_transfer_mode!=1)
// 	{
// 		_set_direction_registers(display_transfer_mode);
// 		P_RS	= portOutputRegister(digitalPinToPort(RS));
// 		B_RS	= digitalPinToBitMask(RS);
// 		P_WR	= portOutputRegister(digitalPinToPort(WR));
// 		B_WR	= digitalPinToBitMask(WR);
// 		P_CS	= portOutputRegister(digitalPinToPort(CS));
// 		B_CS	= digitalPinToBitMask(CS);
// 		P_RST	= portOutputRegister(digitalPinToPort(RST));
// 		B_RST	= digitalPinToBitMask(RST);
// 		if (display_transfer_mode==LATCHED_16)
// 		{
// 			P_ALE	= portOutputRegister(digitalPinToPort(SER));
// 			B_ALE	= digitalPinToBitMask(SER);
// 			cbi(P_ALE, B_ALE);
// 			pinMode(8,OUTPUT);
// 			digitalWrite(8, LOW);
// 		}
// 	}
// 	else
// 	{
// 		P_SDA	= portOutputRegister(digitalPinToPort(RS));
// 		B_SDA	= digitalPinToBitMask(RS);
// 		P_SCL	= portOutputRegister(digitalPinToPort(WR));
// 		B_SCL	= digitalPinToBitMask(WR);
// 		P_CS	= portOutputRegister(digitalPinToPort(CS));
// 		B_CS	= digitalPinToBitMask(CS);
// 		if (RST != NOTINUSE)
// 		{
// 			P_RST	= portOutputRegister(digitalPinToPort(RST));
// 			B_RST	= digitalPinToBitMask(RST);
// 		}
// 		if (display_serial_mode!=SERIAL_4PIN)
// 		{
// 			P_RS	= portOutputRegister(digitalPinToPort(SER));
// 			B_RS	= digitalPinToBitMask(SER);
// 		}
// 	}
// }

uint8_t UTFT::brightnessInPercent = 100;

void UTFT::LCD_Write_COM(uint8_t command)		//ilitek s.31
{   
	DC_TFT_LOW;
	CS_TFT_LOW;
	SPI.SPI_transfer(command);
	CS_TFT_HIGH;
}

void UTFT::LCD_Write_DATA(uint8_t data)			//ilitek s.31
{
	DC_TFT_HIGH;
	CS_TFT_LOW;
	SPI.SPI_transfer(data);
	CS_TFT_HIGH;
}

void UTFT::LCD_Write_DATA(uint8_t VH,uint8_t VL)	//16 Bit dateien
{
	LCD_Write_DATA(VH);								//HighByte
	LCD_Write_DATA(VL);								//LowByte
}

void UTFT::LCD_Write_COM_DATA(uint8_t com1,uint8_t dat1)
{
     LCD_Write_COM(com1);
     LCD_Write_DATA(dat1>>8,dat1);
}

void UTFT::InitLCD(void)
{
// 	 if (_rst > 0) {
//     pinMode(_rst, OUTPUT);
//     digitalWrite(_rst, LOW);
//   }

//   CS_TFT_HIGH;
//   DC_TFT_HIGH;
// 
//   if(hwSPI) { // Using hardware SPI
//     SPI.SPI_init();
// 
// #ifndef SPI_HAS_TRANSACTION
//     SPI1.setBitOrder(MSBFIRST);
//     SPI1.setDataMode(SPI_MODE0);
//   #if defined (_AVR__)
//     SPI1.setClockDivider(SPI_CLOCK_DIV2); // 8 MHz (full! speed!)
//     mySPCR = SPCR;
//   #elif defined(TEENSYDUINO)
//     SPI1.setClockDivider(SPI_CLOCK_DIV2); // 8 MHz (full! speed!)
//   #elif defined (__arm__)
//     SPI1.setClockDivider(11); // 8-ish MHz (full! speed!)
//   #endif
// #endif
//   } else {
//     pinMode(_sclk, OUTPUT);
//     pinMode(_mosi, OUTPUT);
//     pinMode(_miso, INPUT);

// #if defined (USE_FAST_PINIO)
//     clkport     = portOutputRegister(digitalPinToPort(_sclk));
//     clkpinmask  = digitalPinToBitMask(_sclk);
//     mosiport    = portOutputRegister(digitalPinToPort(_mosi));
//     mosipinmask = digitalPinToBitMask(_mosi);
//     *clkport   &= ~clkpinmask;
//     *mosiport  &= ~mosipinmask;
// #endif
  // }

  // toggle RST low to reset
//   if (_rst > 0) {
//     digitalWrite(_rst, HIGH);
//     _delay_ms(5);
//     digitalWrite(_rst, LOW);
//     _delay_ms(20);
//     digitalWrite(_rst, HIGH);
//     _delay_ms(150);
//   }

  /*
  uint8_t x = readcommand8(ILI9341_RDMODE);
  Serial.print("\nDisplay Power Mode: 0x"); Serial.println(x, HEX);
  x = readcommand8(ILI9341_RDMADCTL);
  Serial.print("\nMADCTL Mode: 0x"); Serial.println(x, HEX);
  x = readcommand8(ILI9341_RDPIXFMT);
  Serial.print("\nPixel Format: 0x"); Serial.println(x, HEX);
  x = readcommand8(ILI9341_RDIMGFMT);
  Serial.print("\nImage Format: 0x"); Serial.println(x, HEX);
  x = readcommand8(ILI9341_RDSELFDIAG);
  Serial.print("\nSelf Diagnostic: 0x"); Serial.println(x, HEX);
*/
  //if(cmdList) commandList(cmdList);
  
  //if (hwSPI) spi_begin();
  
  	DDRF |= (1<<PF0);
  	PORTF |= (1<<PF0);

  	TCCR2A = (1 << COM2A1) |  (1 << WGM20);
  	TCCR2A = (1 << CS21);						// clock source = CLK/8, start PWM
  	//setBrightness(255);

  	//Reset Pin 4 (Mega Board: Port G, Pin 5)
  	// Data Direction = output;
  	//DDRG |= 0x20;
  	
  	//CS Pin 5 -- (Mega Borad: port E , pin 3)
  	DDRB |= (1<<PB0);
  	// DC Pin 6 -- (Port H, Pin 3)
  	DDRA |= (1<<PA0);
  	SPI.SPI_init();
  	CS_TFT_HIGH;
  	DC_TFT_HIGH;
  	_delay_ms(100); //alter Wert 100
	  
  LCD_Write_COM(0xEF);
  LCD_Write_DATA(0x03);
  LCD_Write_DATA(0x80);
  LCD_Write_DATA(0x02);

  LCD_Write_COM(0xCF);  
  LCD_Write_DATA(0x00); 
  LCD_Write_DATA(0XC1); 
  LCD_Write_DATA(0X30); 

  LCD_Write_COM(0xED);  
  LCD_Write_DATA(0x64); 
  LCD_Write_DATA(0x03); 
  LCD_Write_DATA(0X12); 
  LCD_Write_DATA(0X81); 
 
  LCD_Write_COM(0xE8);  
  LCD_Write_DATA(0x85); 
  LCD_Write_DATA(0x00); 
  LCD_Write_DATA(0x78); 

  LCD_Write_COM(0xCB);  
  LCD_Write_DATA(0x39); 
  LCD_Write_DATA(0x2C); 
  LCD_Write_DATA(0x00); 
  LCD_Write_DATA(0x34); 
  LCD_Write_DATA(0x02); 
 
  LCD_Write_COM(0xF7);  
  LCD_Write_DATA(0x20); 

  LCD_Write_COM(0xEA);  
  LCD_Write_DATA(0x00); 
  LCD_Write_DATA(0x00); 
 
  LCD_Write_COM(ILI9341_PWCTR1);    //Power control 
  LCD_Write_DATA(0x23);   //VRH[5:0] 
 
  LCD_Write_COM(ILI9341_PWCTR2);    //Power control 
  LCD_Write_DATA(0x10);   //SAP[2:0];BT[3:0] 
 
  LCD_Write_COM(ILI9341_VMCTR1);    //VCM control 
  LCD_Write_DATA(0x3e); //Աȶȵ
  LCD_Write_DATA(0x28); 
  
  LCD_Write_COM(ILI9341_VMCTR2);    //VCM control2 
  LCD_Write_DATA(0x86);  //--
 
  LCD_Write_COM(ILI9341_MADCTL);    // Memory Access Control 
  LCD_Write_DATA(0x48);

  LCD_Write_COM(ILI9341_PIXFMT);    
  LCD_Write_DATA(0x55); 
  
  LCD_Write_COM(ILI9341_FRMCTR1);    
  LCD_Write_DATA(0x00);  
  LCD_Write_DATA(0x18); 
 
  LCD_Write_COM(ILI9341_DFUNCTR);    // Display Function Control 
  LCD_Write_DATA(0x08); 
  LCD_Write_DATA(0x82);
  LCD_Write_DATA(0x27);  
 
  LCD_Write_COM(0xF2);    // 3Gamma Function Disable 
  LCD_Write_DATA(0x00); 
 
  LCD_Write_COM(ILI9341_GAMMASET);    //Gamma curve selected 
  LCD_Write_DATA(0x01); 
 
  LCD_Write_COM(ILI9341_GMCTRP1);    //Set Gamma 
  LCD_Write_DATA(0x0F); 
  LCD_Write_DATA(0x31); 
  LCD_Write_DATA(0x2B); 
  LCD_Write_DATA(0x0C); 
  LCD_Write_DATA(0x0E); 
  LCD_Write_DATA(0x08); 
  LCD_Write_DATA(0x4E); 
  LCD_Write_DATA(0xF1); 
  LCD_Write_DATA(0x37); 
  LCD_Write_DATA(0x07); 
  LCD_Write_DATA(0x10); 
  LCD_Write_DATA(0x03); 
  LCD_Write_DATA(0x0E); 
  LCD_Write_DATA(0x09); 
  LCD_Write_DATA(0x00); 
  
  LCD_Write_COM(ILI9341_GMCTRN1);    //Set Gamma 
  LCD_Write_DATA(0x00); 
  LCD_Write_DATA(0x0E); 
  LCD_Write_DATA(0x14); 
  LCD_Write_DATA(0x03); 
  LCD_Write_DATA(0x11); 
  LCD_Write_DATA(0x07); 
  LCD_Write_DATA(0x31); 
  LCD_Write_DATA(0xC1); 
  LCD_Write_DATA(0x48); 
  LCD_Write_DATA(0x08); 
  LCD_Write_DATA(0x0F); 
  LCD_Write_DATA(0x0C); 
  LCD_Write_DATA(0x31); 
  LCD_Write_DATA(0x36); 
  LCD_Write_DATA(0x0F); 

LCD_Write_COM(0x11);    	//Exit Sleep
_delay_ms(120);

LCD_Write_COM(0x29);    //Display on
LCD_Write_COM(0x2c);

// LCD_Write_COM(0xEF);
// LCD_Write_DATA(0x03);
// LCD_Write_DATA(0x80);
// LCD_Write_DATA(0x02);
// 
// LCD_Write_COM(0xCF);
// LCD_Write_DATA(0x00);
// LCD_Write_DATA(0XC1);
// LCD_Write_DATA(0X30);
// 
// LCD_Write_COM(0xED);
// LCD_Write_DATA(0x64);
// LCD_Write_DATA(0x03);
// LCD_Write_DATA(0X12);
// LCD_Write_DATA(0X81);
// 
// LCD_Write_COM(0xE8);
// LCD_Write_DATA(0x85);
// LCD_Write_DATA(0x00);
// LCD_Write_DATA(0x78);
// 
// LCD_Write_COM(0xCB);
// LCD_Write_DATA(0x39);
// LCD_Write_DATA(0x2C);
// LCD_Write_DATA(0x00);
// LCD_Write_DATA(0x34);
// LCD_Write_DATA(0x02);
// 
// LCD_Write_COM(0xF7);
// LCD_Write_DATA(0x20);
// 
// LCD_Write_COM(0xEA);
// LCD_Write_DATA(0x00);
// LCD_Write_DATA(0x00);
// 
// LCD_Write_COM(ILI9341_PWCTR1);    //Power control
// LCD_Write_DATA(0x23);   //VRH[5:0]
// 
// LCD_Write_COM(ILI9341_PWCTR2);    //Power control
// LCD_Write_DATA(0x10);   //SAP[2:0];BT[3:0]
// 
// LCD_Write_COM(ILI9341_VMCTR1);    //VCM control
// LCD_Write_DATA(0x3e); //Աȶȵ
// LCD_Write_DATA(0x28);
// 
// LCD_Write_COM(ILI9341_VMCTR2);    //VCM control2
// LCD_Write_DATA(0x86);  //--
// 
// LCD_Write_COM(ILI9341_MADCTL);    // Memory Access Control
// LCD_Write_DATA(0x48);
// 
// LCD_Write_COM(ILI9341_PIXFMT);
// LCD_Write_DATA(0x55);
// 
// LCD_Write_COM(ILI9341_FRMCTR1);
// LCD_Write_DATA(0x00);
// LCD_Write_DATA(0x18);
// 
// LCD_Write_COM(ILI9341_DFUNCTR);    // Display Function Control
// LCD_Write_DATA(0x08);
// LCD_Write_DATA(0x82);
// LCD_Write_DATA(0x27);
// 
// LCD_Write_COM(0xF2);    // 3Gamma Function Disable
// LCD_Write_DATA(0x00);
// 
// LCD_Write_COM(ILI9341_GAMMASET);    //Gamma curve selected
// LCD_Write_DATA(0x01);
// 
// LCD_Write_COM(ILI9341_GMCTRP1);    //Set Gamma
// LCD_Write_DATA(0x0F);
// LCD_Write_DATA(0x31);
// LCD_Write_DATA(0x2B);
// LCD_Write_DATA(0x0C);
// LCD_Write_DATA(0x0E);
// LCD_Write_DATA(0x08);
// LCD_Write_DATA(0x4E);
// LCD_Write_DATA(0xF1);
// LCD_Write_DATA(0x37);
// LCD_Write_DATA(0x07);
// LCD_Write_DATA(0x10);
// LCD_Write_DATA(0x03);
// LCD_Write_DATA(0x0E);
// LCD_Write_DATA(0x09);
// LCD_Write_DATA(0x00);
// 
// LCD_Write_COM(ILI9341_GMCTRN1);    //Set Gamma
// LCD_Write_DATA(0x00);
// LCD_Write_DATA(0x0E);
// LCD_Write_DATA(0x14);
// LCD_Write_DATA(0x03);
// LCD_Write_DATA(0x11);
// LCD_Write_DATA(0x07);
// LCD_Write_DATA(0x31);
// LCD_Write_DATA(0xC1);
// LCD_Write_DATA(0x48);
// LCD_Write_DATA(0x08);
// LCD_Write_DATA(0x0F);
// LCD_Write_DATA(0x0C);
// LCD_Write_DATA(0x31);
// LCD_Write_DATA(0x36);
// LCD_Write_DATA(0x0F);
// 
// LCD_Write_COM(ILI9341_SLPOUT);    //Exit Sleep
// //if (hwSPI) spi_end();
// _delay_ms(5);
// //if (hwSPI) spi_begin();
// LCD_Write_COM(ILI9341_DISPON);    //Display on
}

void UTFT::setXY(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)	//ilitek s.35
{
			DC_TFT_LOW;
			CS_TFT_LOW;
			LCD_Write_COM(0x2A); //Spalte
			DC_TFT_HIGH;
			LCD_Write_DATA(x1>>8);
			LCD_Write_DATA(x1);
			LCD_Write_DATA(x2>>8);
			LCD_Write_DATA(x2);
			DC_TFT_LOW;
			LCD_Write_COM(0x2B); //Zeile
			DC_TFT_HIGH;
			LCD_Write_DATA(y1>>8);
			LCD_Write_DATA(y1);
			LCD_Write_DATA(y2>>8);
			LCD_Write_DATA(y2);
			DC_TFT_LOW;
			LCD_Write_COM(0x2C); //Speicher beschreibbar
			CS_TFT_HIGH;
			
	}

void UTFT::clrXY()
{
	if (orient==PORTRAIT)
		setXY(0,0,disp_x_size,disp_y_size);
	else
		setXY(0,0,disp_y_size,disp_x_size);
}

void UTFT::drawRect(int x1, int y1, int x2, int y2)						//x1 & y1: start - x2 & y2 ende
{
	if (x1>x2)
	{
		swap(int, x1, x2);
	}
	if (y1>y2)
	{
		swap(int, y1, y2);
	}

	drawHLine(x1, y1, x2-x1);
	drawHLine(x1, y2, x2-x1);
	drawVLine(x1, y1, y2-y1);
	drawVLine(x2, y1, y2-y1);
}			

void UTFT::drawRoundRect(int x1, int y1, int x2, int y2)
{
	if (x1>x2)
	{
		swap(int, x1, x2);
	}
	if (y1>y2)
	{
		swap(int, y1, y2);
	}
	if ((x2-x1)>4 && (y2-y1)>4)
	{
		drawPixel(x1+1,y1+1);
		drawPixel(x2-1,y1+1);
		drawPixel(x1+1,y2-1);
		drawPixel(x2-1,y2-1);
		drawHLine(x1+2, y1, x2-x1-4);
		drawHLine(x1+2, y2, x2-x1-4);
		drawVLine(x1, y1+2, y2-y1-4);
		drawVLine(x2, y1+2, y2-y1-4);
	}
}

void UTFT::fillRect(int x1, int y1, int x2, int y2)
{
		for (int i=0; i<((y2-y1)/2)+1; i++)
		{
			drawHLine(x1, y1+i, x2-x1);
			drawHLine(x1, y2-i, x2-x1);
		}
}

void UTFT::fillRoundRect(int x1, int y1, int x2, int y2)
{
	if (x1>x2)
	{
		swap(int, x1, x2);
	}
	if (y1>y2)
	{
		swap(int, y1, y2);
	}

	if ((x2-x1)>4 && (y2-y1)>4)
	{
		for (int i=0; i<((y2-y1)/2)+1; i++)
		{
			switch(i)
			{
			case 0:
				drawHLine(x1+2, y1+i, x2-x1-4);
				drawHLine(x1+2, y2-i, x2-x1-4);
				break;
			case 1:
				drawHLine(x1+1, y1+i, x2-x1-2);
				drawHLine(x1+1, y2-i, x2-x1-2);
				break;
			default:
				drawHLine(x1, y1+i, x2-x1);
				drawHLine(x1, y2-i, x2-x1);
			}
		}
	}
}

void UTFT::drawCircle(int x, int y, int radius)
{
	int f = 1 - radius;
	int ddF_x = 1;
	int ddF_y = -2 * radius;
	int x1 = 0;
	int y1 = radius;
 
	cbi(P_CS, B_CS);
	setXY(x, y + radius, x, y + radius);
	LCD_Write_DATA(fch,fcl);
	setXY(x, y - radius, x, y - radius);
	LCD_Write_DATA(fch,fcl);
	setXY(x + radius, y, x + radius, y);
	LCD_Write_DATA(fch,fcl);
	setXY(x - radius, y, x - radius, y);
	LCD_Write_DATA(fch,fcl);
 
	while(x1 < y1)
	{
		if(f >= 0) 
		{
			y1--;
			ddF_y += 2;
			f += ddF_y;
		}
		x1++;
		ddF_x += 2;
		f += ddF_x;    
		setXY(x + x1, y + y1, x + x1, y + y1);
		LCD_Write_DATA(fch,fcl);
		setXY(x - x1, y + y1, x - x1, y + y1);
		LCD_Write_DATA(fch,fcl);
		setXY(x + x1, y - y1, x + x1, y - y1);
		LCD_Write_DATA(fch,fcl);
		setXY(x - x1, y - y1, x - x1, y - y1);
		LCD_Write_DATA(fch,fcl);
		setXY(x + y1, y + x1, x + y1, y + x1);
		LCD_Write_DATA(fch,fcl);
		setXY(x - y1, y + x1, x - y1, y + x1);
		LCD_Write_DATA(fch,fcl);
		setXY(x + y1, y - x1, x + y1, y - x1);
		LCD_Write_DATA(fch,fcl);
		setXY(x - y1, y - x1, x - y1, y - x1);
		LCD_Write_DATA(fch,fcl);
	}
	sbi(P_CS, B_CS);
	clrXY();
}

void UTFT::fillCircle(int x, int y, int radius)
{
	for(int y1=-radius; y1<=0; y1++) 
		for(int x1=-radius; x1<=0; x1++)
			if(x1*x1+y1*y1 <= radius*radius) 
			{
				drawHLine(x+x1, y+y1, 2*(-x1));
				drawHLine(x+x1, y-y1, 2*(-x1));
				break;
			}
}

void UTFT::clrScr(void)
{
	// zugriff auf komplettes Display
	setXY(0,0,239,319);

	DC_TFT_HIGH;
	CS_TFT_LOW;

	// pro Pixel 2 bytes (320*240 = 76800)
	for(uint16_t i=0; i<38400; i++)
	{
		SPI.SPI_transfer(0);
		SPI.SPI_transfer(0);
		SPI.SPI_transfer(0);
		SPI.SPI_transfer(0);
	}
	
	CS_TFT_HIGH;
}

void UTFT::fillScr(byte r, byte g, byte b)
{
	uint16_t color = ((r&248)<<8 | (g&252)<<3 | (b&248)>>3);
	fillScr(color);
}

void UTFT::fillScr(uint16_t color)
{

// zugriff auf komplettes Display
setXY(0,0,239,319);

DC_TFT_HIGH;
CS_TFT_LOW;


for (long i=0; i<76800; i++)
{

	SPI.SPI_transfer(color>>8);
	SPI.SPI_transfer(color&0xff);
}
CS_TFT_HIGH;
}

void UTFT::setColor(byte r, byte g, byte b)
{
	fch=((r&248)|g>>5);
	fcl=((g&28)<<3|b>>3);
}

void UTFT::setColor(uint16_t color)
{
	fch=byte(color>>8);
	fcl=byte(color & 0xFF);
}

uint16_t UTFT::getColor()
{
	
	return (fch<<8) | fcl;
}

void UTFT::setBackColor(byte r, byte g, byte b)
{
	bch=((r&248)|g>>5);
	bcl=((g&28)<<3|b>>3);
	_transparent=false;
}

void UTFT::setBackColor(uint32_t color)
{
		bch=byte(color>>8);
		bcl=byte(color & 0xFF);
}

uint16_t UTFT::getBackColor()
{
	return (bch<<8) | bcl;
}

// void UTFT::setPixel(word color)
// {
// 	LCD_Write_DATA((color>>8),(color&0xFF));	// rrrrrggggggbbbbb
// }

void UTFT::drawPixel(int16_t x, int16_t y)
{
	setXY(x, y, x, y);
	LCD_Write_DATA(fch, fcl);
}

void UTFT::drawLine(int x1, int y1, int x2, int y2)
{
	if (y1==y2)
		drawHLine(x1, y1, x2-x1);
	else if (x1==x2)
		drawVLine(x1, y1, y2-y1);
	else
	{
		unsigned int	dx = (x2 > x1 ? x2 - x1 : x1 - x2);
		short			xstep =  x2 > x1 ? 1 : -1;
		unsigned int	dy = (y2 > y1 ? y2 - y1 : y1 - y2);
		short			ystep =  y2 > y1 ? 1 : -1;
		int				col = x1, row = y1;

		cbi(P_CS, B_CS);
		if (dx < dy)
		{
			int t = - (dy >> 1);
			while (true)
			{
				setXY (col, row, col, row);
				LCD_Write_DATA (fch, fcl);
				if (row == y2)
					return;
				row += ystep;
				t += dx;
				if (t >= 0)
				{
					col += xstep;
					t   -= dy;
				}
			} 
		}
		else
		{
			int t = - (dx >> 1);
			while (true)
			{
				setXY (col, row, col, row);
				LCD_Write_DATA (fch, fcl);
				if (col == x2)
					return;
				col += xstep;
				t += dy;
				if (t >= 0)
				{
					row += ystep;
					t   -= dx;
				}
			} 
		}
		sbi(P_CS, B_CS);
	}
	clrXY();
}

void UTFT::drawHLine(int x, int y, int l)
{
	if (l<0)
	{
		l = -l;
		x -= l;
	}
	cbi(P_CS, B_CS);
	setXY(x, y, x+l, y);
	DC_TFT_HIGH;
	CS_TFT_LOW;
		for (int i=0; i<l+1; i++)
		{
			LCD_Write_DATA(fch, fcl);
		}
	
	CS_TFT_HIGH;
	//clrXY();
}

void UTFT::drawVLine(int x, int y, int l)
{
	if (l<0)
	{
		l = -l;
		y -= l;
	}
	cbi(P_CS, B_CS);
	setXY(x, y, x, y+l);
	DC_TFT_HIGH;
	CS_TFT_LOW;
		for (int i=0; i<l+1; i++)
		{
			LCD_Write_DATA(fch, fcl);
		}
	CS_TFT_HIGH;
	//clrXY();
}


void UTFT::printNumI(long num, int x, int y, int length, char filler)		//print an integer at the specified coordinates
																			//you can use LEFT,CENTER, RIGHT as the x- coordinate to align the string on the screen
{
	char buf[25];
	char st[27];
	bool neg=false;
	int c=0, f=0;
  
	if (num==0)
	{
		if (length!=0)
		{
			for (c=0; c<(length-1); c++)
				st[c]=filler;
			st[c]=48;
			st[c+1]=0;
		}
		else
		{
			st[0]=48;
			st[1]=0;
		}
	}
	else
	{
		if (num<0)
		{
			neg=true;
			num=-num;
		}
	  
		while (num>0)
		{
			buf[c]=48+(num % 10);
			c++;
			num=(num-(num % 10))/10;
		}
		buf[c]=0;
	  
		if (neg)
		{
			st[0]=45;
		}
	  
		if (length>(c+neg))
		{
			for (int i=0; i<(length-c-neg); i++)
			{
				st[i+neg]=filler;
				f++;
			}
		}

		for (int i=0; i<c; i++)
		{
			st[i+neg+f]=buf[c-i-1];
		}
		st[c+neg+f]=0;

	}

	print(st,x,y);
}

void UTFT::printNumF(double num, byte dec, int x, int y, char divider, int length, char filler)		//print a floating-point number at the specified coordinates
{
	char st[27];
	bool neg=false;

	if (dec<1)
		dec=1;
	else if (dec>5)
		dec=5;

	if (num<0)
		neg = true;

	_convert_float(st, num, length, dec);

	if (divider != '.')
	{
		for (int i=0; i<sizeof(st); i++)
			if (st[i]=='.')
				st[i]=divider;
	}

	if (filler != ' ')
	{
		if (neg)
		{
			st[0]='-';
			for (int i=1; i<sizeof(st); i++)
				if ((st[i]==' ') || (st[i]=='-'))
					st[i]=filler;
		}
		else
		{
			for (int i=0; i<sizeof(st); i++)
				if (st[i]==' ')
					st[i]=filler;
		}
	}

	print(st,x,y);
}   

void UTFT::setFont(uint8_t* font)
{
	cfont.font=font;
	cfont.x_size=fontbyte(0);
	cfont.y_size=fontbyte(1);
	cfont.offset=fontbyte(2);
	cfont.numchars=fontbyte(3);
}

uint8_t* UTFT::getFont()
{
	return cfont.font;
}

uint8_t UTFT::getFontXsize()
{
	return cfont.x_size;
}

uint8_t UTFT::getFontYsize()
{
	return cfont.y_size;
}

void UTFT::drawBitmap(int x, int y, int sx, int sy, bitmapdatatype data, int scale)
{
	unsigned int col;
	int tx, ty, tc, tsx, tsy;
	byte r, g, b;

	if (scale==1)
	{
		setXY(x, y, x+sx-1, y+sy-1);
		for (tc=0; tc<(sx*sy); tc++)
		{
			col=pgm_read_word(&data[tc]);
			LCD_Write_DATA(col>>8,col & 0xff);
		}
	}
	else
	{
		for (ty=0; ty<sy; ty++)
		{
			setXY(x, y+(ty*scale), x+((sx*scale)-1), y+(ty*scale)+scale);
			for (tsy=0; tsy<scale; tsy++)
			for (tx=0; tx<sx; tx++)
			{
				col=pgm_read_word(&data[(ty*sx)+tx]);
				for (tsx=0; tsx<scale; tsx++)
				LCD_Write_DATA(col>>8,col & 0xff);
			}
		}
	}
	
	//clrXY();
}


int UTFT::getDisplayXSize()
{
	if (orient==PORTRAIT)
		return disp_x_size+1;
	else
		return disp_y_size+1;
}


 void UTFT::setBrightness(uint8_t br)
 {
	if (br>100)
	{
		brightnessInPercent = 100;
	}
	else if (br<5)
	{
		brightnessInPercent = 5;
	}
	else
	{
		brightnessInPercent = br;
	}
}

//UTFT myGLCD;


